@@ -36,6 +36,13 @@ from snakebite.version import version
 from snakebite.namenode import Namenode
 
 
+def print_error_exit(msg, fd=sys.stderr):
+    print >> fd, "Error: %s" % msg
+    sys.exit(-1)
+
+def print_info(msg, fd=sys.stderr):
+    print >> fd, "Info: %s" % msg
+
 def exitError(error):
     if isinstance(error, FileNotFoundException) or \
        isinstance(error, DirectoryException) or \
@@ -100,7 +107,7 @@ class CommandLineParser(object):
                           "type": float},
                     'p': {"short": '-p',
                           "long": '--port',
-                          "help": 'namenode RPC port',
+                          "help": 'namenode RPC port (default: %d)' % Namenode.DEFAULT_PORT,
                           "type": int},
                     'h': {"short": '-h',
                           "long": '--help',
@@ -124,6 +131,15 @@ class CommandLineParser(object):
                       "long": '--summary',
                       "help": 'print summarized output',
                       "action": 'store_true'},
+                'S': {"short": '-S',
+                      "long": '--skiptrash',
+                      "help": 'skip the trash (when trash is enabled)',
+                      "default": False,
+                      "action": 'store_true'},
+                'T': {"short": '-T',
+                      "long": "--usetrash",
+                      "help": "enable the trash",
+                      "action": 'store_true'},
                 'z': {"short": '-z',
                       "long": '--zero',
                       "help": 'check for zero length',
@@ -225,6 +241,28 @@ class CommandLineParser(object):
             command_parser = subparsers.add_parser(cmd_name, add_help=False, parents=parents)
             command_parser.set_defaults(command=cmd_name)
 
+    def init(self):
+        self.read_config()
+        self._clean_args()
+        self.setup_client()
+
+    def _clean_args(self):
+        for path in self.__get_all_directories():
+            if path.startswith('hdfs://'):
+                parse_result = urlparse(path)
+                if path in self.args.dir:
+                    self.args.dir.remove(path)
+                    self.args.dir.append(parse_result.path)
+                else:
+                    self.args.single_arg = parse_result.path
+
+    def __usetrash_unset(self):
+        return not 'usetrash' in self.args or self.args.usetrash == False
+
+    def __use_cl_port_first(self, alt):
+        # Port provided from CL has the highest priority:
+        return self.args.port if self.args.port else alt
+
     def read_config(self):
 
         # Try to retrieve namenode config from within CL arguments
@@ -243,16 +281,14 @@ class CommandLineParser(object):
         else:
             # Try to read the configuration for HDFS configuration files
             configs = HDFSConfig.get_external_config()
-
             # if configs exist and contain something
             if configs:
                 for config in configs:
                     nn = Namenode(config['namenode'],
-                                  config['port'])
+                                  self.__use_cl_port_first(config['port']))
                     self.namenodes.append(nn)
-
-                # Save retrieved configuration to snakebite config file
-                self._write_hadoop_config(config_file)
+                if self.__usetrash_unset():
+                    self.args.usetrash = HDFSConfig.use_trash
 
         if len(self.namenodes):
             return
@@ -265,77 +301,106 @@ class CommandLineParser(object):
             for hdfs_conf_path in HDFSConfig.hdfs_try_paths:
                 print " - %s" % hdfs_conf_path
             print "\nYou can manually create ~/.snakebiterc with the following content:"
-            print '{"namenode": "ip/hostname", "port": 54310, "version": %d}' % Namenode.DEFAULT_VERSION
+            print '{'
+            print '  "config_version": 2,'
+            print '  "use_trash": true,'
+            print '  "namenodes": ['
+            print '    {"host": "namenode-ha1", "port": %d, "version": %d},' % (Namenode.DEFAULT_PORT, Namenode.DEFAULT_VERSION)
+            print '    {"host": "namenode-ha2", "port": %d, "version": %d}' % (Namenode.DEFAULT_PORT, Namenode.DEFAULT_VERSION)
+            print '  ]'
+            print '}'
+
             sys.exit(1)
 
     def _read_config_snakebiterc(self):
+        old_version_info = "You're are using snakebite %s with Trash support together with old snakebiterc, please update/remove your ~/.snakebiterc file. By default Trash is %s." % (version(), 'disabled' if not HDFSConfig.use_trash else 'enabled')
         with open(os.path.join(os.path.expanduser('~'), '.snakebiterc')) as config_file:
             configs = json.load(config_file)
 
         if isinstance(configs, list):
+            # Version 1: List of namenodes
             # config is a list of namenode(s) - possibly HA
             for config in configs:
                 nn = Namenode(config['namenode'],
-                              config['port'],
+                              self.__use_cl_port_first(config.get('port', Namenode.DEFAULT_PORT)),
                               config.get('version', Namenode.DEFAULT_VERSION))
                 self.namenodes.append(nn)
+            if self.__usetrash_unset():
+                # commandline setting has higher priority
+                print_info(old_version_info)
+                # There's no info about Trash in version 1, use default policy:
+                self.args.usetrash = HDFSConfig.use_trash
         elif isinstance(configs, dict):
-             # config is a single namenode - no HA
-            self.namenodes.append(Namenode(configs['namenode'],
-                                           configs['port'],
-                                           configs.get('version', Namenode.DEFAULT_VERSION)))
-        else:
-            print "Config retrieved from ~/.snakebiterc is corrupted! Remove it!"
-            sys.exit(1)
+            # Version 2: {}
+            # Can be either new configuration or just one namenode
+            # which was the very first configuration syntax
+            if 'config_version' in configs:
+                # Config version => 2
+                for nn_config in configs['namenodes']:
+                    nn = Namenode(nn_config['host'],
+                                  self.__use_cl_port_first(nn_config.get('port', Namenode.DEFAULT_PORT)),
+                                  nn_config.get('version', Namenode.DEFAULT_VERSION))
+                    self.namenodes.append(nn)
 
-    def _read_config_cl(self):
-        ''' Check if any directory arguments contain hdfs://'''
+                if self.__usetrash_unset():
+                    # commandline setting has higher priority
+                    self.args.usetrash = configs.get("use_trash", HDFSConfig.use_trash)
+            else:
+                # config is a single namenode - no HA
+                self.namenodes.append(Namenode(configs['namenode'],
+                                               self.__use_cl_port_first(configs.get('port', Namenode.DEFAULT_PORT)),
+                                               configs.get('version', Namenode.DEFAULT_VERSION)))
+                if self.__usetrash_unset():
+                    # commandline setting has higher priority
+                    print_info(old_version_info)
+                    self.args.usetrash = HDFSConfig.use_trash
+        else:
+            print_error_exit("Config retrieved from ~/.snakebiterc is corrupted! Remove it!")
+
+    def __get_all_directories(self):
         if self.args and 'dir' in self.args:
             dirs_to_check = list(self.args.dir)
             if self.args.command == 'mv':
                 dirs_to_check.append(self.args.single_arg)
-            for directory in dirs_to_check:
-                if 'hdfs://' in directory:
-                    parse_result = urlparse(directory)
+            return dirs_to_check
+        else:
+            return ()
 
-                    if not self.args.namenode is None and not self.args.port is None and (self.args.port != parse_result.port or self.args.namenode != parse_result.hostname):
-                        print "error: conflicting nodenames or ports"
-                        sys.exit(-1)
-                    else:
-                        self.args.namenode = parse_result.hostname
-                        self.args.port = parse_result.port
+    def _read_config_cl(self):
+        ''' Check if any directory arguments contain hdfs://'''
+        dirs_to_check = self.__get_all_directories()
+        hosts, ports = [], []
+        for path in dirs_to_check:
+            if path.startswith('hdfs://'):
+                parse_result = urlparse(path)
+                hosts.append(parse_result.hostname)
+                ports.append(parse_result.port)
 
-                        if directory in self.args.dir:
-                            self.args.dir.remove(directory)
-                            self.args.dir.append(parse_result.path)
-                        else:
-                            self.args.single_arg = parse_result.path
+        # remove duplicates and None from (hosts + self.args.namenode)
+        hosts = filter(lambda x: x != None, set(hosts + [self.args.namenode]))
+        if len(hosts) > 1:
+            print_error_exit('Conficiting namenode hosts in commandline arguments, hosts: %s' % str(hosts))
 
-        if self.args.namenode and self.args.port:
-            # If namenode config found based on arguments, save namenode
+        ports = filter(lambda x: x != None, set(ports + [self.args.port]))
+        if len(ports) > 1:
+            print_error_exit('Conflicting namenode ports in commandline arguments, ports: %s' % str(ports))
+
+        # Store port from CL in arguments - CL port has the highest priority
+        if len(ports) == 1:
+            self.args.port = ports[0]
+
+        # do we agree on one namenode?
+        if len(hosts) == 1 and len(ports) <= 1:
+            self.args.namenode = hosts[0]
+            self.args.port = ports[0] if len(ports) == 1 else Namenode.DEFAULT_PORT
             self.namenodes.append(Namenode(self.args.namenode, self.args.port))
+            # we got the info from CL -> check if use_trash is set - if not use default policy:
+            if self.__usetrash_unset():
+                self.args.usetrash = HDFSConfig.use_trash
             return True
         else:
             return False
 
-    def _write_hadoop_config(self, config_file_path):
-        # Write config to file
-        with open(config_file_path, "w") as config_file:
-            config = []
-            for namenode in self.namenodes:
-                config.append(namenode.toDict())
-
-            if len(config) > 1:
-                #If many NNs use list syntax
-                pretty_config = json.dumps(config).split('},')
-                config_file.write('},\n'.join(pretty_config))
-            elif len(config) == 1:
-                #If just one NN use old syntax
-                config_file.write(json.dumps(config[0]))
-            else:
-                print "Try to write configuration without NameNode configuration!"
-                sys.exit(-1)
-
     def parse(self, non_cli_input=None):  # Allow input for testing purposes
         if not sys.argv[1:] and not non_cli_input:
             self.parser.print_help()
@@ -364,7 +429,11 @@ class CommandLineParser(object):
         return self.args
 
     def setup_client(self):
-        self.client = HAClient(self.namenodes)
+        if 'skiptrash' in self.args:
+            use_trash = self.args.usetrash and not self.args.skiptrash
+        else:
+            use_trash = self.args.usetrash
+        self.client = HAClient(self.namenodes, use_trash)
 
     def execute(self):
         if self.args.help:
@@ -501,7 +570,7 @@ class CommandLineParser(object):
         for line in format_results(result, json_output=self.args.json):
             print line
 
-    @command(args="[paths]", descr="remove paths", allowed_opts=["R"], req_args=['dir [dirs]'])
+    @command(args="[paths]", descr="remove paths", allowed_opts=["R", "S", "T"], req_args=['dir [dirs]'])
     def rm(self):
         result = self.client.delete(self.args.dir, recurse=self.args.recurse)
         for line in format_results(result, json_output=self.args.json):
